// C. Herzog fuer Grundlagen der Programmierung, 1.12.2004

class ArraySet extends Set {

// Attribute (Datenstruktur):

    private static final int DEFAULT_CAPACITY = 5;
    private static final int DEFAULT_CAPACITY_INCREMENT = 5;
    private int currentSize;		// aktuelle Groesse der Menge
    private Object[] array;		// speichert die Elemente der Menge

// verschiedene Konstruktoren fuer eine leere Menge:

    // gewuenschte Reihungskapazitaet wird uebergeben
    public ArraySet(int capacity) {
        array = new Object[capacity];
	currentSize = 0;
    }

    // Reihungskapazitaet richtet sich nach dem Default-Wert
    ArraySet() {
        this(DEFAULT_CAPACITY);
    }

// Konstruktor, der die Kopie einer Menge liefert:

    // die Reihungsgroesse wird so gewaehlt, dass zusaetzliche
    // Elemente Platz finden
    public ArraySet(Set s) {
	currentSize = s.size();

	if (currentSize < DEFAULT_CAPACITY)
	    array = new Object[DEFAULT_CAPACITY];
	else
	    array = new Object[currentSize + DEFAULT_CAPACITY_INCREMENT];

        // die Elemente aus s werden uebertragen:
	int index=0;
	Enumeration enum = s.getEnumeration();
	while (enum.hasMoreElements()) {
	   array[index] = enum.nextElement();
	   index++;
        }
    }

// sonstige datenstrukturabhaengige Methoden:

    // liefert Enumeration ueber die Menge:
    public Enumeration getEnumeration() {
        return new ArraySetEnumeration(currentSize, array);
    }

    // Einfuegen eines Elementes:
    public void insert(Object o) {

	// o darf noch nicht enthalten sein:
	if (contains(o)) {
	    System.out.println("insert: " + o + " schon enthalten!");
	    return;
	}

	// wenn neues Element nicht hineinpasst:
	if (currentSize >= array.length) {

	    // alte Reihung zwischenspeichern:
            Object[] oldArray = array;

            // Neue Reihung anlegen:
            array = new Object[currentSize + 1 + DEFAULT_CAPACITY_INCREMENT];
            
            // Umspeichern der Elemente:
            for (int index=0; index<currentSize; index++)
		array[index] = oldArray[index];
	}

        // Speichern von o auf letztem Platz:
        array[currentSize] = o;

        // Konsistente Erhoehung von currentSize:
        currentSize++;

    }

    // Entfernen eines Elementes:
    public void delete(Object o) {

	// Indexposition von o ermitteln:
	int index = 0;
	while (index < currentSize && ! array[index].equals(o))
	   index++;

	if (index >= currentSize) {
	    // in diesem Fall ist o nicht in Menge enthalten
	    System.out.println("delete: " + o + " nicht enthalten!");
	    return;
	}

	// Sonst steht o auf Position index; o wird geloescht, indem
	// das rechteste Element auf Position index umgespeichert wird
        array[index] = array[currentSize-1];

        // Konsistente Verminderung von currentSize:
        currentSize--;

    }
}
