// C. Herzog fuer Grundlagen der Programmierung, 1.12.2004

class OrderedArraySet extends OrderedSet {

// Attribute (Datenstruktur):

    private static final int DEFAULT_CAPACITY = 5;
    private static final int DEFAULT_CAPACITY_INCREMENT = 5;
    private int currentSize;		// aktuelle Groesse der Menge
    private Comparable[] array;	// speichert die Elemente der Menge

// verschiedene Konstruktoren fuer eine leere Menge:

    // gewuenschte Reihungskapazitaet wird uebergeben
    public OrderedArraySet(int capacity) {
        array = new Comparable[capacity];
	currentSize = 0;
    }

    // Reihungskapazitaet richtet sich nach dem Default-Wert
    OrderedArraySet() {
        this(DEFAULT_CAPACITY);
    }

// Konstruktor, der die Kopie einer Menge liefert:

    // die Reihungsgroesse wird jeweils so gewaehlt, dass zusaetzliche
    // Elemente Platz finden

    // Zuerst ein Konstruktor speziell fr sortierte Mengen:
    public OrderedArraySet(OrderedSet s) {

	currentSize = s.size();

	if (currentSize < DEFAULT_CAPACITY)
	    array = new Comparable[DEFAULT_CAPACITY];
	else
	    array = new Comparable[currentSize + DEFAULT_CAPACITY_INCREMENT];

        // ausgenutzt wird, dass die Elemente in s sortiert sind:
        int index=0;
	Enumeration enum = s.getEnumeration();
	while (enum.hasMoreElements()) {
	   array[index] = (Comparable) enum.nextElement();
	   index++;
	}
    }

    //  Sonst muss sich der Konstruktur auf insert abstuetzen:
    public OrderedArraySet(Set s) {

        if (s.size() < DEFAULT_CAPACITY)
	    array = new Comparable[DEFAULT_CAPACITY];
	else
	    array = new Comparable[s.size() + DEFAULT_CAPACITY_INCREMENT];

        currentSize = 0; // vorerst ist Menge leer

	Enumeration enum = s.getEnumeration();
	while (enum.hasMoreElements())
	   insert(enum.nextElement());
    }

// sonstige datenstrukturabhaengige Methoden:

    // effiziente Abfrage nach Groesse der Menge:
    public int size() {

	return currentSize;
    }

    // liefert Enumeration ueber die Menge:
    public Enumeration getEnumeration() {
        return new ArraySetEnumeration(currentSize, array);
    }

    // Einfuegen eines Elementes:
    public void insert(Object o) {

	Comparable c = (Comparable) o;

	// Zunaechst Suche nach der Einfuegestelle:
	int index = 0;
	while (index < currentSize && array[index].compareTo(c) == -1)
	   index++;

	// c darf noch nicht enthalten sein:
	if (index < currentSize && array[index].compareTo(c) == 0) {
	    System.out.println("insert: " + c + " schon enthalten!");
	    return;
	}

	// wenn ein neues Element noch hineinpasst:
	if (currentSize < array.length) {

            // Verschieben der restlichen Elemente nach rechts:
            for (int k=currentSize-1; k>=index; k--)
		array[k+1] = array[k];
	}
	else
	{   // im anderen Fall muss eine groessere Reihung angelegt werden:

	    // alte Reihung zwischenspeichern:
            Comparable[] oldArray = array;

            // Neue Reihung anlegen:
            array = new Comparable[currentSize + 1 + DEFAULT_CAPACITY_INCREMENT];
            
            // Umspeichern der vorne liegenden Elemente:
            for (int k=0; k<index; k++)
		array[k] = oldArray[k];
            
            // Umspeichern der hinten liegenden Elemente mit Luecke:
            for (int k=index; k<currentSize; k++)
		array[k+1] = oldArray[k];
	}

        // Speichern von c auf Position index:
	array[index] = c;
            
        // Konsistente Erhoehung von currentSize:
        currentSize++;

    }

    // Entfernen eines Elementes:
    public void delete(Object o) {

	Comparable c = (Comparable) o;

	// Indexposition von c ermitteln:
	int index = 0;
	while (index < currentSize && array[index].compareTo(c) == -1)
	   index++;

	if (index >= currentSize || array[index].compareTo(c) == 1) {
	    // in diesem Fall ist c nicht in Menge enthalten
	    System.out.println("delete: " + c + " nicht enthalten!");
	    return;
	}

	// Sonst steht c auf Position index; c wird geloescht, indem
	// die Elemente rechts von Position index nach links umgespeichert
	// werden
	for (int k=index+1; k<currentSize; k++)
           array[k-1] = array[k];

        // Konsistente Verminderung von currentSize:
        currentSize--;

    }
}
